(function() {
	if (!window.azumio) {
		window.azumio = {};
	}

	if (!window.azumio.timeline) {
		window.azumio.timeline = {};
	}

	var timeline = window.azumio.timeline;

	timeline.messages = {};
	
	timeline.messages.intro = function(){
		var note = {type:'azumio'};
		
		note.note = "<p>We are pleased to introduce the Timeline feature, which allows you to quickly store and view your measurements.</p>"
			+"<p>Timeline also offers you an aggregated view of data across all Azumio apps, including Stress Check.</p>"
			+"<p>In the future, you will receive recommendations and useful hints for your personal health through Azumio Timeline entries.</p>"
			+"<p>We hope you will enjoy the new Azumio experience!</p>"
			+"Best, Team Azumio";
		return note;
	}; 

	
	timeline.load = function() {
		if (timeline.isLoading)
			return;
		timeline.isLoading = true;

		var uncommited = azumio.getUncommitedCheckins();

		timeline.onTimelineLoaded(uncommited);

		var fake = timeline.messages.intro();

		

		timeline.showLoadingIndicator();
		azumio.getCheckins(function(checkins){
				if(checkins.length < 2){
					timeline.append([fake]);
				}
				timeline.onTimelineLoaded(checkins);
			}, null, function(e) {
			timeline.showLoadingError(e);
			timeline.isLoading = false;
		});
	}

	timeline.lastEntry = null;
	timeline.isLoading = false;
	timeline.loadMore = function() {
		if (timeline.isLoading)
			return;
		timeline.isLoading = true;

		timeline.showLoadingIndicator();

		var filter = {};
		if (timeline.lastEntry) {
			filter.beforeTimestamp = timeline.lastEntry.timestamp;
		}
		azumio.getCheckins(timeline.onTimelineLoaded, filter, function(e) {
			timeline.showLoadingError(e);
			timeline.isLoading = false;
		});
	}

	timeline.onTimelineLoaded = function(entries) {
		timeline.isLoading = false;
		timeline.hideLoadingIndicator();

		if (entries && entries.length > 0) {
			timeline.append(entries);
			timeline.lastEntry = entries[entries.length - 1];
		}
	}

	timeline.showLoadingIndicator = function() {
		$(".listLoader").removeClass("reload reloadError loading unauthorized");
		$(".listLoader").addClass("loading");
	}

	timeline.hideLoadingIndicator = function() {
		$(".listLoader").removeClass("reload reloadError loading unauthorized");
		$(".listLoader").addClass("reload");
	}

	timeline.showLoadingError = function(e) {
		$(".listLoader").removeClass("reload reloadError loading unauthorized");

		if (e && e.meta && e.meta.code.toString().indexOf("401") == 0) {
			$(".listLoader").addClass("unauthorized");
		} else {
			$(".listLoader").addClass("reloadError");
		}
	}
	var counter = 0;

	timeline.createEntry = function(data) {
		timeline.addToCache(data);

		var checkin = data;
		var newCheckin = timeline.templates.timelineEntry.clone();

		newCheckin.attr("id", null);
		newCheckin.attr("checkinId", checkin.id);
		newCheckin.attr("checkinRemoteId", checkin.remoteid);

		if (!checkin.id) {
			newCheckin.addClass("unsynced");
		}

		if (typeof checkin.confirmed != undefined && checkin.confirmed == false) {
			newCheckin.addClass("unconfirmed");
		}

		var type = checkin.type;
		var value = checkin.value;
		var note = checkin.note;

		var typeInfo = types[type];
		// var typeInfo = {};

		newCheckin.find("img.icon").attr("src", "img/types/" + type + ".png");
		// newCheckin.find(".type").append(types[type].title);
		if (typeInfo && typeInfo.title) {
			newCheckin.find(".type").append(typeInfo.title);
		} else {
			newCheckin.find(".type").remove();
		}

		if (value) {
			newCheckin.find(".value").append(" - " + value);
		} else {
			newCheckin.find(".value").remove();
		}

		if (typeInfo && typeInfo.unit) {
			newCheckin.find(".unit").append(typeInfo.unit);
		} else {
			newCheckin.find(".unit").remove();
		}
		newCheckin.find(".note").append(note);

		// FIXME:

		var time = azumio.utils.timespanToString(checkin.timestamp);

		newCheckin.find(".time").append(time);
		newCheckin.find(".time").attr("timestamp", checkin.timestamp);

		if (checkin.location) {
			newCheckin.find(".locationMarker").show();
		} else {
			newCheckin.find(".locationMarker").hide();
		}

		newCheckin.find(".content").click(function(event) {
			if (checkin.type == 'heartrate') {
				timeline.editEntry(newCheckin);
				azumio.trackEvent("timeline.edit");
			} else {
				//alert(checkin.client_id);
			}
		});
		
		
		newCheckin.find(" .delete").click(function(event) {
			azumio.timeline.deleteEntry(newCheckin,checkin);
		});
		
		newCheckin.find(".content").bind("swipe",function(event) {
			//alert("swipe");
			var deleteButton = newCheckin.find("button.delete");
			var visible = deleteButton.hasClass("visible");
			
			$("button.delete.visible").removeClass("visible");
			
			if( visible )
				deleteButton.removeClass("visible");
			else
				deleteButton.addClass("visible");
			
		});
		
		

		return newCheckin;
	}
	var scrollTopBeforeEdit = 0;
	timeline.editEntry = function(entry) {
		
		scrollTopBeforeEdit = $("body").scrollTop();
		// $(".entry.edit").each(function(a){
		// $(this).removeClass("edit");
		// $(this).find(".newentry").remove();
		// });

		var checkin = {};
	
		if (entry.attr("checkinId") ) {
			checkin = timeline.getFromCache(entry.attr("checkinId"))
		} else if (entry.attr("checkinRemoteId")) {
			checkin = timeline.getFromCache(entry.attr("checkinRemoteId"))
		}

		// alert(JSON.stringify(checkin));

		var animationHolder = null;
		// var editor = timeline.templates.editor.clone();

		var te = $("#timelineeditor");
		// te.append(editor);

		var editor = $("#timelineeditor .newentry");

		$("#timelineHolder").addClass("editing");
		// editor.css("top","-400px");
		// editor.animate({top:0});

		timeline.prepareEditor(editor, checkin)

		// prevent scrolling
		$("#content").css("height", (window.innerHeight) + "px");

		return;



	}

	timeline.cache = {};
	timeline.getFromCache = function(id) {
		return timeline.cache[id];
	}

	timeline.addToCache = function(checkin) {
		if (checkin.remoteid)
			timeline.cache[checkin.remoteid] = checkin;
		if (checkin.id)
			timeline.cache[checkin.id] = checkin;
	}

	timeline.append = function(list) {
		var ul = $("#timeline");
		for ( var i = 0; i < list.length; i++) {
			lastCheckin = list[i];
			var li = timeline.createEntry(list[i]);
			{
				var e = [];
				if( e.length == 0 && lastCheckin.remoteid){
					e = $(".entry[checkinRemoteId='" + lastCheckin.remoteid + "']");
				}
				
				if( lastCheckin.id  ){
					e = $(".entry[checkinId='" + lastCheckin.id + "']");
				}
				
				if(e.length == 0){
					ul.append(li);
				}
			}
			
		}
	}

	timeline.prependEntry = function(entry) {
		var ul = $("#timeline");
		var li = timeline.createEntry(entry);
		ul.prepend(li);
		return li;
	}

	timeline.handlers = {};

	timeline.handlers.save = function() {
		azumio.trackEvent("timeline.save");
		
		var form = $(this).parents(".newentry");
		var entry;

		var checkin = {};
		if (form.attr("checkinId")) {
			checkin = timeline.getFromCache(form.attr("checkinId"));
			entry = $("#timeline .entry[checkinId='" + form.attr("checkinId")
					+ "']");
		} else if (form.attr("checkinRemoteId")) {
			checkin = timeline.getFromCache(form.attr("checkinRemoteId"));
			entry = $("#timeline .entry[checkinRemoteId='"
					+ form.attr("checkinRemoteId") + "']");
		}

		delete checkin.attachments;

		var checkin = azumio.createCheckinFromForm(form, checkin);
		checkin.confirmed = true;

		azumio.addCheckin(checkin);
		azumio.sync();

		entry.find(".content").html(
				timeline.createEntry(checkin).find('.content').html());

		timeline.endEditing();
		if(!azumio.auth.isAuthenticated()){
			azumio.auth.showLogin();
		}
	}

	timeline.handlers.back = function() {
		// var entry = $(this).parents(".entry");
		// var form = $(this).parents(".newentry");
		// form.remove();
		timeline.endEditing();
	}

	timeline.handlers.deleteClick = function() {
		
		azumio.trackEvent("timeline.delete");
		
		var form = $(this).parents(".newentry");
		azumio.utils.confirm("Delete?", function() {

			var entry;

			var checkin = {};
			if (form.attr("checkinId")) {
				checkin = timeline.getFromCache(form.attr("checkinId"));
				entry = $("#timeline .entry[checkinId='"
						+ form.attr("checkinId") + "']");
			} else if (form.attr("checkinRemoteId")) {
				checkin = timeline.getFromCache(form.attr("checkinRemoteId"));
				entry = $("#timeline .entry[checkinRemoteId='"
						+ form.attr("checkinRemoteId") + "']");
			}

			delete checkin.attachments;

			checkin.deleted = true;

			azumio.addCheckin(checkin);
			azumio.sync();

			

			
			

			var top = entry.position().top;
			
			entry.animate({opacity:0.1},1000,function(){entry.remove();});
			
			timeline.endEditing();
			
//			setTimeout(function() {
//				$('html, body').animate({
//					scrollTop : top + 30
//				}, 1);
//			});

		});
	}
	
	
	timeline.deleteEntry = function(entry,checkin) {
		
		azumio.trackEvent("timeline.deleteEntry");
		
		azumio.utils.confirm("Delete?", function() {



			delete checkin.attachments;

			checkin.deleted = true;

			azumio.addCheckin(checkin);
			azumio.sync();

			entry.animate({opacity:0.1},1000,function(){entry.remove();});
			
			timeline.endEditing();
		});
	}

	timeline.endEditing = function() {
		$("#timelineHolder").removeClass("editing");
		$("#content").css("height", "");
		
		setTimeout(function() {
			$("body").scrollTop(scrollTopBeforeEdit);
		},1);
		
		azumio.locationServicesOff();
	}

	timeline.bindEditorEvents = function(editor) {
		editor.find("button.save").click(timeline.handlers.save);
		editor.find("button.delete").click(timeline.handlers.deleteClick);
		editor.find(".back").click(timeline.handlers.back);

		editor.find(".social.facebook").click(function() {
			// alert("Share on facebook")
		});

		editor.find(".social.twitter").click(function() {
			// alert("Share on twitter")
		});

		editor.find(".tag").click(function() {
			var note = editor.find(".note input");
			var text = note.val();
			var tag = $(this).html();
			if (text.indexOf(tag) >= 0) {
				text = text.replace(" " + tag, "");
				text = text.replace(tag, "");
			} else {
				text = text + " " + tag;
			}
			note.val(text);
		});

		editor.find("input[name='note']").click(function(e) {
			var dist = e.srcElement.clientWidth - e.offsetX;
			if (dist < 50) {
				$(this).val($(this).val() + " #");
			}
		});

		editor.find(".share-buttons button").click(function(e) {
			$(this).toggleClass("checked");
		});

		// editor.get(0).addEventListener('touchmove', preventDefaultScroll,
		// false);
	}

	timeline.prepareEditor = function(editor, checkin) {

		timeline.addToCache(checkin);

		var value = editor.find(".value");
		value.html(checkin.value);

		editor.find("img.icon").attr("src",
				"img/types/" + checkin.type + ".png");

		if(checkin.id)
			editor.attr("checkinId", checkin.id);
		else
			editor.attr("checkinId",null);
		
		if(checkin.remoteid)
			editor.attr("checkinRemoteId", checkin.remoteid);
		else
			editor.attr("checkinRemoteId", null);

		editor.find("input[name=value]").val(checkin.value);

		editor.find("input[name=note]").val(checkin.note);
		editor.find("span.value").html(checkin.value);
		editor.find("input[name=remoteid]").val(checkin.remoteid);

		var time = azumio.utils.timespanToString(checkin.timestamp);
		editor.find(".time").append(time);
		editor.find(".time").attr("timestamp", checkin.timestamp);

		if (checkin.location) {
			editor.find(".locationMarker").show();
		} else {
			editor.find(".locationMarker").hide();
		}

		editor.find(".share-buttons button").removeClass("checked");

		if (checkin.broadcast) {
			if (checkin.broadcast.indexOf("twitter") >= 0) {
				editor.find(".share-buttons button.twitter")
						.addClass("checked");
			}

			if (checkin.broadcast.indexOf("facebook") >= 0) {
				editor.find(".share-buttons button.facebook").addClass(
						"checked");
			}
		}

		timeline.prepareEditor
				.movePin(editor.find('.resultPin'), checkin.value);
		editor.find('.resultValue').html(checkin.value);

		if (typeof checkin.confirmed != "undefined"
				&& checkin.confirmed == false) {
			editor.addClass("unconfirmed");
		} else {
			editor.removeClass("unconfirmed");
		}
	}

	timeline.prepareEditor.movePin = function(pin, val) {
		var p = {
			w : 540,
			ls : 20,
			le : 184,
			ms : 192,
			me : 348,
			hs : 356 - 120,
			he : 520 - 120
		};

		var r = {
			r0 : 40,
			r1 : 60,
			r2 : 90,
			r3 : 120
		};
		var pos = 0;
		if (val < r.r1) {
			pos = p.ls + (p.le - p.ls) * Math.max(val - r.r0, 0)
					/ (r.r1 - r.r0);
		}

		if (val >= r.r1 && val <= r.r2) {
			pos = p.ms + (p.me - p.ms) * Math.max(val - r.r1, 0)
					/ (r.r2 - r.r1);
		}

		if (val > r.r2) {
			pos = p.hs + (p.he - p.hs) * Math.min(val - r.r2, r.r3 - r.r2)
					/ (r.r3 - r.r2);
			pin.addClass("reversed");
		} else {
			pin.removeClass("reversed");
		}
		pos = pos / 2;
		pin.css("margin-left", pos + "px");
	}

	timeline.clean = function() {
		$('.edit2').children().unwrap();
		$('#timeline  .newentry').remove();
	}

	timeline.templates = {};
	timeline.loadTemplates = function() {
		timeline.templates.animationHolder = $(".templates.animationHolder")
				.children().clone();
		timeline.templates.editor = $(".templates.timelineEntryEditor")
				.children().clone();
		timeline.templates.timelineEntry = $(".templates.timelineEntry")
				.children().clone();
		;
	}
	
	
	timeline.migrate = function(){
		if(localStorage["timeline.migrated"]){
			return;
		}
		
		$(window).bind("azumio.history",function(e,data){
			var checkins = data.checkins;
			timeline.append(checkins);
			for ( var i = 0; i < checkins.length; i++) {
				var checkin = checkins[i];
				azumio.addCheckin(checkin);
			}
			
			azumio.sync();
			localStorage["timeline.migrated"] = true;
		});	
		
		azumio.exec("azumio:requestHistory:");
	}

	timeline.init = function() {

		try {
			timeline.loadTemplates();

			{
				var editor = timeline.templates.editor.clone();
				var te = $("#timelineeditor");
				te.append(editor);
				timeline.bindEditorEvents(editor);
			}

			$(window).bind("azumio.timeline.update", function(event, entry) {
				var e = [];
				if( entry.id ){
					e = $(".entry[checkinId='" + entry.id + "']");
				}
				if (e.length == 0 && entry.remoteid)
					e = $(".entry[checkinRemoteId='" + entry.remoteid + "']");
				
				e.removeClass("unsynced");
			});

			$(".listLoader .reload").click(function() {
				timeline.loadMore();
			});

			$(".listLoader .reloadError").click(function() {
				timeline.loadMore();
			});

			$(".listLoader .login").click(function() {
				azumio.auth.showLogin();
			});

			$(window).bind("azumio.auth.login", function() {
				timeline.loadMore();
			});

			var profiler = {};
			profiler.times = [];
			profiler.mark = function() {
				profiler.times.push(new Date().getTime());
			}
			profiler.report = function() {
				var out = [];
				var i;
				var t = profiler.times;
				for (i = 1; i < profiler.times.length; i++) {
					out.push(t[i] - t[i - 1]);
				}

				return JSON.stringify(out);
			}
			profiler.reset = function() {
				profiler.times = [];
			}

			// show result section
			var counter = 0;
			$(window).bind("azumio.newresult", function(event, data) {
				azumio.getLocation();

				azumio.trackEvent('timeline.newresult',{value:""+data.value});
				
				setTimeout(function() {
					counter = counter + 1;

					timeline.clean();
					var checkin = {
						value : data.value,
						type : 'heartrate',
						remoteid : azumio.newId(),
						timestamp : (new Date()).getTime(),
						confirmed : false
					};

					azumio.attachFilesToCheckin(checkin);

					azumio.addCheckin(checkin);
					azumio.sync();

					var entry = timeline.prependEntry(checkin);
					timeline.editEntry(entry);

				}, 1);

			});

			$(window).bind("scrollend", function() {
				timeline.loadMore();
			});
		} catch (e) {

		}
		
		timeline.migrate();
		timeline.load();
	}
	
	$(timeline.init);
})();
